//
//  Green.cpp
//  DMFT-IPT
//
//  Created by Shaheen Acheche on 29/09/2016.
//  Copyright © 2016 Shaheen Acheche. All rights reserved.
//

#include "Green.hpp"


using namespace std;
typedef complex<double> dcomp;
typedef vector<dcomp> cd_vector;

Green_function::Green_function(){
    init = false;
    Matsubara=true;
    this->g_Nmin = 0;
    this->g_Nmax =0;
}

Green_function::Green_function(int provbeta, int Nmax, cd_vector rest, int Nmin)
{   beta=provbeta;
    dcomp I(0.0, 1.0);
    for(int i=Nmin; i<Nmax; i++){
        iwn.push_back(I*2.0*M_PI*(i +0.5)/double(beta));
        G.push_back(1.0/(iwn[i]+rest[i]));
    }
    init = true;
    Matsubara=true;
}

Green_function::Green_function(int beta, int Nmax, int Nmin){
    this->beta = beta;
    this->g_Nmax= Nmax;
    this->g_Nmin = Nmin;
    dcomp I(0.0, 1.0);
    for(int i =Nmin; i<Nmax; i++){
        this->iwn.push_back(I*2.0*M_PI*(i +0.5)/double(beta));
        this->tau.push_back((i-Nmin)*beta/double(Nmax-Nmin));
    }
    init = false;
    Matsubara=true;
}

Green_function::Green_function(int provbeta,  int Nmax, cd_vector rest, double mu, int Nmin)
{
    dcomp I(0.0,1.0);
    beta=provbeta;
    for(int i=0; i<(Nmax-Nmin); i++){
        iwn.push_back((2*M_PI*I*(i +0.5)/double(beta)));
        G.push_back(1.0/(iwn[i]+rest[i]));
    }
    init = true;
}

Green_function Green_function::operator=(Green_function const& b)
{
    this->beta = b.get_beta();
    this->iwn = b.get_iwn();
    this->G = b.get_G();
    this->mu = b.get_mu();
    this->init =true;
    this->g_Nmin = b.get_Nmin();
    this->g_Nmax = b.get_Nmax();
    this->tau = b.get_tau();
    return *this; 
}

void Green_function::get_new_by_adding(cd_vector rest, double gmu){
    if(rest.size() != this->size()){
        cout<<"ERROR, the vector used does not have the same size as the Green function"<<endl;
        cout<<"size of vector ="<<rest.size()<<" and size of Green function ="<<this->size()<<endl;
    }
    this->G.clear();
    for(int i =0; i<iwn.size(); i++){
        G.push_back(arrond_0(1.0/(iwn[i]-rest[i]+gmu)));

    }
    this->update_mu(gmu);
    init = true;
    
}

void Green_function::get_new(cd_vector rest){
    this->G.clear();
    for(int i=0; i<this->size(); i++){
        G.push_back((1./(rest[i]+this->mu)));
      //  G.push_back((rest[i]+this->mu));

    }
    this->init=true;
}


Green_function Green_function::operator+(Green_function const& b){
    if(b.init==false){
        cout<<"ERROR. Green's function not defined"<<endl;
    }
    Green_function prov(b.get_beta(), b.get_Nmax(), b.get_Nmin());
    prov.update_mu(this->mu);
    for(int i =0; i< b.size();i++){
        prov.G.push_back(this->G[i]+ b.G[i]);
    }
    return prov;
}

Green_function Green_function::operator-(Green_function const& b){
    if(b.init==false){
        cout<<"ERROR. Green's function not defined"<<endl;
    }
    Green_function prov(b.get_beta(), b.get_Nmax(), b.get_Nmin());
    prov.update_mu(this->mu);
    for(int i =0; i< this->size();i++){
        prov.G.push_back(this->G[i]- b.G[i]);
    }
    return prov;

}
Green_function Green_function::operator*(Green_function const& b){
    if(b.init==false){
        cout<<"ERROR. Green's function not defined"<<endl;
    }
    Green_function prov(b.get_beta(), b.get_Nmax(), b.get_Nmax());
    prov.update_mu(this->mu);
    for(int i =0; i<b.size(); i++){
        prov.G.push_back(this->G[i]*b.G[i]);
    }
    return prov;
    
}


void Green_function::get_G_inf()
{
    if(this->is_init()==true){
        cout<<"ERROR. Green's function already defined"<<endl;
    }
    this->G.clear();
    for(int i =0; i<this->size(); i++){
        this->G.push_back(1.0/(iwn[i]+this->mu-this->c/iwn[i]));
    }
    this->init= true;
}

cd_vector Green_function::make_fft_tau_positif()
{
    int taille= this->size();
    Green_function temp(this->beta, this->get_Nmax(), this->get_Nmin()), F(this->beta, this->get_Nmax(), this->get_Nmin());
    temp.update_mu(this->get_mu());
    temp.get_G_inf();
    F= *this - temp;
    double F_decal[2*taille];
    double F_sens[2*taille];
    double j;
    double t;
    int i;
    
    cd_vector resultat;
    dcomp I(0.0,1.0);
    
    for(int i = 0; i<taille; i++){
        F_decal[2*i]=F.iw(i).real();
        F_decal[2*i+1] = F.iw(i).imag();
    }

    gsl_fft_complex_radix2_forward(F_decal, 1, taille);
 for(int i =0; i<taille; i++)
    {
        j= i*F.get_beta()/double(taille);
        //resultat.push_back(F_sens[2*i]+I*F_sens[2*i+1]);
        resultat.push_back(arrond_0(exp(-M_PI*i*I*(1.0/(double(taille))-1))*1./double(F.get_beta())*(F_decal[2*i] +I*F_decal[2*i+1])+Hsup(j, mu, c, F.get_beta())));
        
    }
  //  cout<<"taille fonction de Green="<<G.size()<<endl; +Hsup(j, mu, c, F.get_beta()))
//arrond_0(exp(-M_PI*i*I*(1.0/(double(taille))-1))*1./double(F.get_beta())*(F_decal[2*i] +I*F_decal[2*i+1]))
    this->Matsubara=false;
    return resultat;

}

cd_vector Green_function::make_fft_tau_negatif()
{
    int taille= this->size();
    Green_function temp(this->beta, this->get_Nmax(), this->get_Nmin()), F(this->beta, this->get_Nmax(), this->get_Nmin());
    temp.update_mu(this->get_mu());
    temp.get_G_inf();
    F= *this - temp;
    double F_decal[2*taille];
    int i;
    double j;
    cd_vector resultat;
    dcomp I(0.0,1.0);

    
    for(int i = 0; i<taille; i++)
    {
        F_decal[2*i]=F.iw(i).real();
        F_decal[2*i+1] = F.iw(i).imag();
    }
    gsl_fft_complex_radix2_backward(F_decal, 1, taille);
    

    
    for(int i =0; i<taille; i++)
    {
        j= i*F.get_beta()/double(taille);
        resultat.push_back(arrond_0(exp(M_PI*i*I*(1.0/(double(taille))-1))*1./double(F.get_beta())*(F_decal[2*i] +I*F_decal[2*i+1])+Hinf(j, mu, c, F.get_beta())));

    }
    this->Matsubara=false;
    return resultat;
}


cd_vector Green_function::get_dg_dtau_positif()
{
    int taille= this->size();
    Green_function temp(this->beta, this->get_Nmax(), this->get_Nmin()), F(this->beta, this->get_Nmax(), this->get_Nmin());
    temp.update_mu(this->get_mu());
    temp.get_G_inf();
    F= *this - temp;
    double F_decal[2*taille];
    int i;
    double j;
    cd_vector resultat;
    dcomp I(0.0,1.0);
    
    for(int i = 0; i<taille; i++){
        F_decal[2*i]= iwn[i].imag()*F.iw(i).imag();
        F_decal[2*i+1] = -1.0*iwn[i].imag()*F.iw(i).real();
    }
    gsl_fft_complex_radix2_forward(F_decal, 1, taille);
    for(int i =0; i<taille; i++)
    {
        j= i*F.get_beta()/double(taille);
        resultat.push_back(arrond_0(exp(-M_PI*i*I*(1/(double(taille))-1))*1./double(F.get_beta())*(F_decal[2*i] +I*F_decal[2*i+1])+Hpsup(j, mu, c, F.get_beta())));
    }
    return resultat;
}
cd_vector Green_function::get_dg_dtau_negatif()
{
    int taille= this->size();
    Green_function temp(this->beta, this->get_Nmax(), this->get_Nmin()), F(this->beta, this->get_Nmax(), this->get_Nmin());

    temp.update_mu(this->get_mu());
    temp.get_G_inf();
    F= *this - temp;
    double F_decal[2*taille];
    double j;
    int i;
    cd_vector resultat;
    dcomp I(0.0,1.0);
    
    for(int i = 0; i<taille; i++){
        F_decal[2*i]= -iwn[i].imag()*F.iw(i).imag();
        F_decal[2*i+1] = 1.0*iwn[i].imag()*F.iw(i).real();
    }
    gsl_fft_complex_radix2_backward(F_decal, 1, taille);
    for(int i =0; i<taille; i++)
    {
        j= i*F.get_beta()/double(taille);
        resultat.push_back(arrond_0(exp(M_PI*i*I*(1/(double(taille))-1))*1./double(F.get_beta())*(F_decal[2*i] +I*F_decal[2*i+1])+Hpinf(j, mu, c, F.get_beta())));
    }
    return resultat;
}

d_vector Green_function::get_tau() const{
    return this->tau;
}


int Green_function::size() const
{
    return g_Nmax-g_Nmin;
}

int Green_function::get_beta() const
{
    return beta;
}

int Green_function::get_Nmax() const
{
    return g_Nmax;
}

int Green_function::get_Nmin() const
{
    return g_Nmin;
}
dcomp Green_function::iw(int i) const
{
    return G[i];
}

cd_vector Green_function::get_iwn() const
{
    return iwn;
}
cd_vector Green_function::get_G() const
{
    return this->G;
}

bool Green_function::is_init(){
    return init;
}


double Green_function::get_mu() const{
    return this->mu;
}
void Green_function::update_mu(double gmu){
    mu = gmu;
}


double nF(double x, double beta) {
    return 1./(exp(beta*x)+1);
}

double Hsup(double tau, double mu, double c, int beta){
    dcomp H;
    double z1(-0.5*mu + 0.5*sqrt(mu*mu+4*c));
    double z2(-0.5*mu-0.5*sqrt(mu*mu+4*c));
    
    return -z1/(z1-z2)*1./(exp(z1*(tau-double(beta)))+exp(z1*tau)) - z2/(z2-z1)*1./(exp(z2*(tau-double(beta)))+exp(z2*tau));
}

double Hpsup(double tau, double mu, double c, int beta){
    double z1(-0.5*mu + 0.5*sqrt(mu*mu+4*c));
    double z2(-0.5*mu-0.5*sqrt(mu*mu+4*c));
    return z1*z1/(z1-z2)*1./(exp(z1*(tau-double(beta)))+exp(z1*tau)) + z2*z2/(z2-z1)*1./(exp(z2*(tau-double(beta)))+exp(z2*tau));
}

double Hinf(double tau, double mu, double c, int beta){
    dcomp H;
    double z1(-0.5*mu + 0.5*sqrt(mu*mu+4*c));
    double z2(-0.5*mu-0.5*sqrt(mu*mu+4*c));
    
    return z1/(z1-z2)*1.0/(exp(z1*(double(beta)-tau))+exp(-z1*tau)) + z2/(z2-z1)*1./(exp(z2*(double(beta)-tau))+exp(-z2*tau));
}




double Hpinf(double tau, double mu, double c, int beta){
    dcomp H;
    double z1(-0.5*mu + 0.5*sqrt(mu*mu+4*c));
    double z2(-0.5*mu-0.5*sqrt(mu*mu+4*c));
    
    return z1*z1/(z1-z2)*1.0/(exp(z1*(double(beta)-tau))+exp(-z1*tau))  + z2*z2/(z2-z1)*1./(exp(z2*(double(beta)-tau))+exp(-z2*tau));
}





