//
//  Green.hpp
//  DMFT-IPT
//
//  Created by Shaheen Acheche on 29/09/2016.
//  Copyright © 2016 Shaheen Acheche. All rights reserved.
//

#ifndef Green_hpp
#define Green_hpp

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <complex>
#include <ctime>
#include <vector>
#include "integrator.hpp"
#include <gsl/gsl_errno.h>
#include <gsl/gsl_fft_complex.h>

typedef std::complex<double> dcomp;
typedef std::vector<dcomp> cd_vector;
typedef std::vector<double> d_vector;

class Green_function{
public:
    Green_function();
    Green_function(int beta, int Nmax=100, int Nmin=0);
    Green_function(int provbeta, int Nmax, cd_vector rest,  int Nmin=0);
    Green_function(int provbeta, int Nmax, cd_vector rest, double mu =0, int Nmin=0);
    Green_function operator+(Green_function const& b);
    Green_function operator-(Green_function const& b);
    Green_function operator=(Green_function const& b);
    Green_function operator*(Green_function const& b);

    cd_vector make_fft_tau_positif();
    cd_vector make_fft_tau_negatif();
    cd_vector get_dg_dtau_positif();
    cd_vector get_dg_dtau_negatif();

    void update_mu(double gmu);
    void get_new_by_adding(cd_vector rest, double gmu =0);
    void get_new(cd_vector rest);
    void get_G_inf();
    cd_vector get_iwn() const;
    int get_beta() const;
    dcomp iw(int i) const;
    bool is_init();
    int size() const;
    double get_mu() const;
    int get_Nmax() const;
    int get_Nmin() const;
    cd_vector get_G() const;
    d_vector get_tau() const;
    
private:
    int beta;
    double c=4;
    int g_Nmin;
    int g_Nmax;
    cd_vector G;
    cd_vector iwn;
    d_vector tau;
    double mu=0;
    bool init;
    bool Matsubara;
};
double Hsup(double tau, double mu, double c, int beta);
double Hpsup(double tau, double mu, double c, int beta);
double Hpinf(double tau, double mu, double c, int beta);
double Hinf(double tau, double mu, double c, int beta);
double nF(double x, double beta);
#endif /* Green_hpp */
