//
//  Spline.cpp
//  Spline_test
//
//  Created by Shaheen Acheche on 05/10/2016.
//  Copyright © 2016 Shaheen Acheche. All rights reserved.
//

#include "Spline.hpp"


using namespace std;

Spline::Spline(){
    this->Sp10=0;
    this->S10 = 0;
    this->Spp10 =0;
    this->SNbeta=0;
    this->SpNbeta =0;
    this->SppNbeta =0;
    this->init = false;
}

void Spline::define(){
    const int taille = G_0.size();
    cd_vector a, b, c(taille+1), d, y;
    cd_vector cp, dp;
    double h;
    int i(0);
    h = G_0.get_tau()[1]-G_0.get_tau()[0];
    
    for(i=0; i<taille+1;i++)
    {   if(i==0){
            y.push_back((3.0*(this->I[1]-this->I[0])/h-3.0*this->derive[0]));
    }else{
        if(i==taille){
            y.push_back((3.0*this->derive[1]-3.0*(this->I[taille]-this->I[taille-1])/h));
        }
        y.push_back((3.0*(this->I[i+1]-this->I[i])/h -3.0*(this->I[i]-this->I[i-1])/h));
        
    }
        
    }
    
    
    for(i=0;i<taille;i++)
    {
        if(i==0){
            cp.push_back(0.5);
        }else{
            cp.push_back(h/(4*(h)-h*cp.back()));
        }
    }
    for(i=0;i<taille+1;i++)
    {
        if(i==0){
            dp.push_back(y[0]/(2*h));
        }
        else{
            if(i==taille){
                dp.push_back((y[i]-h*dp.back())/(2.0*h-h*cp[i-1]));
            }
            else{
                dp.push_back((y[i]-h*dp.back())/(4.0*h-h*cp[i-1]));
            }
        }
    }
    
    for(i=0;i<taille+1;i++)
    {   if(i==0){
        c[taille]=dp.back();
    }else{
        c[taille-i]=dp[taille-i]-cp[taille-i]*c[taille+1-i];
    }
    }
    for(i=0;i<taille;i++){
        a.push_back(this->I[i]);
        d.push_back((c[i+1]-c[i])/(3.0*h));
        b.push_back((this->I[i+1]-this->I[i])/h-c[i]*h-d.back()*h*h);
    }
    this->S10 = a[0];
    this->Sp10 = b[0];
    this->Spp10 = 2.0*c[0];
    this->SNbeta = a.back()+b.back()*h+c[taille-1]*h*h+d.back()*h*h*h;
    
    this->SpNbeta = b.back()+2.0*c[taille-1]*h+3.0*d.back()*h*h;
    this->SppNbeta = 2.0*c[taille-1]+6.0*d.back()*h;
    this->Sppp.clear();
    for(int i=0; i<taille; i++){
        Sppp.push_back(6.0*d[i]);
    }
    this->init=true;
   // cout<<"done"<<endl;
}




cd_vector Spline::compute_Iw(){

    if(this->init ==false){
        cout<<"ERROR! Spline not defined. Please define it before using this function"<<endl;
    }
    int taille= G_0.size();
    double F_decal[2*taille];
    double F_sens[2*taille];
    double j;
    int i;
    //cout<<"taille="<<this->Sppp.size()<<endl;
    cd_vector resultat;
    dcomp I(0.0, 1.0);
    for(i =0; i<taille; i++){
        F_decal[2*i] = (exp(I*M_PI*double(i)/double(taille))*Sppp[i]).real();
        F_decal[2*i+1] = (exp(I*M_PI*double(i)/double(taille))*Sppp[i]).imag();
    }
    gsl_fft_complex_radix2_backward(F_decal, 1, taille);
    
    for(i =0; i<taille; i++){
        if(i<taille/2){
        F_sens[2*i] = F_decal[taille+2*i];
        F_sens[2*i+1] = F_decal[taille+2*i+1];
        }else{
            F_sens[2*i]= F_decal[2*i-taille];
            F_sens[2*i+1] = F_decal[2*i+1-taille];
        }
    }
    for(i = 0; i<taille; i++){
        resultat.push_back(-1.0*arrond_0(-(S10+SNbeta)/G_0.get_iwn()[i] +(Sp10+SpNbeta)/(G_0.get_iwn()[i]*G_0.get_iwn()[i])-(Spp10+SppNbeta)/(G_0.get_iwn()[i]*G_0.get_iwn()[i]*G_0.get_iwn()[i])+(1.0-exp(1.0*G_0.get_iwn()[i]*double(G_0.get_beta())/double(taille)))/(G_0.get_iwn()[i]*G_0.get_iwn()[i]*G_0.get_iwn()[i]*G_0.get_iwn()[i])*(F_sens[2*i]+I*F_sens[2*i+1])));
    }
    return resultat;
}

void Spline::define_G_0(Green_function G, double U){
    this->G_0 = G;
    this->U = U;
    this->init = false;
    
}

void Spline::get_derive(d_comp pos0, d_comp neg0)
{

    cd_vector G0p, dG0p;
    cd_vector G0n, dG0n;
    dG0p=G_0.get_dg_dtau_positif();
    dG0n = G_0.get_dg_dtau_negatif();
    this->derive.clear();
    this->derive.push_back(2.0*pos0*neg0*dG0p[0]+pos0*pos0*dG0n[0]);
    this->derive.push_back(2.0*(-1.0-pos0)*(1.0-neg0)*(-1.0*G_0.get_mu()-dG0p[0])+(-1.0-pos0)*(-1.0-pos0)*(-1.0*G_0.get_mu()-dG0n[0]));
}


void Spline::create_I_tau(){

    cd_vector pos, neg;
    pos= G_0.make_fft_tau_positif();
    neg = G_0.make_fft_tau_negatif();
    this->I.clear();
    for(int i =0; i< G_0.size(); i++){
 
    this->I.push_back(pos[i]*pos[i]*neg[i]);
    }
    this->I.push_back((-1.0-pos[0])*(-1.0-pos[0])*(1.0-neg[0]));

    this->get_derive(pos[0], neg[0]);
    cout<<this->init<<endl;
}

cd_vector Spline::get_I() const{
    return this->I; 
}

d_comp Spline::get_S10() const{
    return this->S10;
}
d_comp Spline::get_Sp10() const{
    return this->Sp10;
}
d_comp Spline::get_Spp10() const{
    return this->Spp10;
}
d_comp Spline::get_SNbeta() const{
    return this->SNbeta;
}
d_comp Spline::get_SpNbeta() const{
    return this->SpNbeta;
}
d_comp Spline::get_SppNbeta() const{
    return this->SppNbeta;
}
cd_vector Spline::get_Sppp() const{
    return this->Sppp;
}