//
//  integrator.cpp
//  DMFT-IPT
//
//  Created by Shaheen Acheche on 28/09/2016.
//  Copyright © 2016 Shaheen Acheche. All rights reserved.
//
#include "integrator.hpp"

dcomp integrator(dcomp (*f)(double, double, dcomp), double a, double b, double c, double d, dcomp complex, double error)
{
    dcomp res(0.0, 0.0);
    dcomp inter(10.0, 10.0);
    dcomp I(0.,1.0);
    dcomp A;
    dcomp B;
    double AR;
    double AI;
    double BR(0.);
    double BI(0.);
    int N = 9;
    bool conv=true;
    const int Max(11);
    int i(0);
    double Dx = (b-a)/N;
    double Dy = (d-c)/N;
    if (error <= 0.0){
        cout<<"ERROR! Error wanted too low or negative. Default error choosen 0.0001\n";
        error = 0.0001;
    }
    while(i< Max && conv){
        AI=0.0;
        AR=0.;
        BR = 0.0;
        BI=0.0;
       // cout<<"N= "<<N<<endl;
        inter = res;
        Dx = (b-a)/N;
        Dy = (d-c)/N;
       // cout<<"number of interation "<<i<<"!\n";
       #pragma omp parallel for reduction(+:AR, AI) private(A)
        for(int j =1; j <N; j++){
            A = f(a+j*Dx, c, complex)+f(a+j*Dx, d, complex)+f(a,c+ j*Dy, complex)+f(b,c+ j*Dy, complex);
            AR+=A.real();
            AI+=A.imag();
        }
        A= AR+I*AI;
       #pragma omp parallel for reduction(+:BR, BI) private(B) collapse(2)
        for(int j = 1; j<N; j++){
            for(int k=1; k<N; k++){
                B = f(a+j*Dx,c+k*Dy, complex);
                BR += B.real();
                BI += B.imag();
            }
        }
        B = BR+I*BI;
        //cout<<"A= "<<A<<" and B= "<<B<<endl;
        res = 0.25*Dx*Dy*(f(a, c, complex)+f(a, d, complex)+f(b, c, complex)+f(b, d, complex))+0.5*Dx*Dy*A+Dx*Dy*B;
        //cout<<"resultat= "<<res<<endl;
        if(sqrt((inter.real()-res.real())*(inter.real()-res.real())+(inter.imag()-res.imag())*(inter.imag()-res.imag()))<error){
            conv = false;
        }
        N = N*2;
        i++;
    }
    if(i ==Max){
        cout<<"WARNING! Convergence not achieved!\n";
        cout<<"last error (real)="<<abs(inter.real()-res.real())<<endl;
        cout<<"last error (imag)="<<abs(inter.imag()-res.imag())<<endl;

    }
    return arrond_0(res);
}



double density(Green_function G)
{
    double n(0);
    for(int i=0; i<G.size();i++)
    {
        n+=G.iw(i).real();
    }
    n=n/double(G.get_beta());
    n+=0.5;
    return n;
}

double density_2ndmethod(double mu, Green_function G,cd_vector denom)
{
    Green_function G_temp(G.get_beta(), G.get_Nmax(), G.get_Nmin());
    G_temp.update_mu(0.0);
    cd_vector temp;
    int i(0);
    for(i=0; i<G.size(); i++){
        temp.push_back(denom[i]+mu);
    }
    G_temp.get_new(temp);
    return density(G_temp);
}







double density_2ndmethod_asymp(double mu, Green_function G,cd_vector denom){
    Green_function G_temp(G.get_beta(), G.get_Nmax(), G.get_Nmin());
    cd_vector temp;
    int i(0);
    for(i=0; i<G.size(); i++){
        temp.push_back(denom[i]+mu);
    }
    G_temp.get_new(temp);
    return density_asymp(G_temp);
}





dcomp arrond_0(dcomp n){
    dcomp I(0.0, 1.0);
    if(abs(n.imag())<=0.000001 && abs(n.real())>0.000001){
        return n.real();
    }
    if(abs(n.real())<=0.000001 && abs(n.imag())>0.000001){
        return I*n.imag();
    }
    if(abs(n.real())<=0.000001 && abs(n.imag())<=0.000001){
        return 0.0;
    }

    return n;

}


double double_occu(cd_vector Self, Green_function G, double n, double U){
    dcomp D=0;
    cd_vector E;
    for(int i= 0; i<Self.size(); i++){
        E.push_back(Self[i]-0.5*U*n);
        D += E[i]*G.iw(i);
    }
    D= D/(U*double(G.get_beta()));
    D += 0.25*n*n;
    return D.real();
}

cd_vector get_Hyb(int iter){
    cd_vector temp;
    dcomp I(0.0, 1.0);
    double a, b, c;
    ifstream lect("Hyb"+to_string(iter)+".txt");
    while(lect>>a>>b>>c){
        temp.push_back(b + I*c);
    }
    return temp;
}


cd_vector Compute_self(cd_vector I, double U, double n, double mu, double mu0){
    cd_vector Self;
    double B;
    B = ((1-0.5*n)*U+mu0-mu)/(0.5*n*(1-0.5*n));
    for(int i = 0; i<I.size(); i++){
        Self.push_back(0.5*U*n + U*U*I[i]/(1.0-B*I[i]));
    }
    return Self;
}

cd_vector Init_hyb(double c, cd_vector iwn)
{
    cd_vector Hyb;
    dcomp I(0.0, 1.0);
    for(int i =0; i<iwn.size(); i++){
        Hyb.push_back(c/(iwn[i]));
           }
    return Hyb;
}


d_vector linspace(double min, double max, int nbr, bool endpoint){
    d_vector d;
    if(nbr==1){
      d.push_back(min);
      return d;
    }
    if(min==max){
      d.push_back(max);
      return d;
    }
    if(endpoint){
    for(int i=0; i<nbr; i++){
        d.push_back(min +i*(max-min)/double((nbr-1.)));
    } 
    }
    else{
    for(int i=0; i<nbr; i++){
        d.push_back(min +i*(max-min)/double((nbr)));
    }
    }
    return d;
}

double density_asymp(Green_function G){
    dcomp n(0.0);
    for(int i =0; i<G.size();i++){
        n += G.iw(i) -1./G.get_iwn()[i];
    }
    n = n/double(G.get_beta());
    return n.real()+0.5;
}

dcomp Green_1(double x, double y, dcomp res){
    return 1.0/((4*M_PI*M_PI)*(res+2*(cos(x)+cos(y))));
}


double False_pos_get_mu_density(Green_function G, cd_vector denom, double n_target, double mu_guess)
{
    double mu_i = -20;
    double mu_f = 20.;
    double mu_third;
    double prec=0.001;
    double a, b,c;
    int compteur(0);
    int max(200);
    a = density_2ndmethod_asymp(mu_i,G, denom)-n_target;
    b = density_2ndmethod_asymp(mu_f,G, denom)-n_target;

    if(a*b>0){
        if(abs(a)<prec){
            return mu_i;
        }
        cout<<"ERROR! Impossible to find mu"<<endl;
        cout<<"Pour mu=0, n="<<a<<endl;
        cout<<"Pour mu=50, n="<<b<<endl;
        return mu_i;
    }
    while(abs(a-b)>prec && compteur <max){
        mu_third = mu_i -(mu_i-mu_f)*a/(a-b);
        //cout<<mu_i<<"   "<<mu_third<<endl;
        c = density_2ndmethod(mu_third,G, denom)-n_target;
        if(a*c>0){
            mu_i = mu_third;
        }
        else{
            mu_f = mu_third;
        }
        a = density_2ndmethod(mu_i,G, denom)-n_target;
        b = density_2ndmethod(mu_f,G, denom)-n_target;
        compteur++;
    }
    cout<<"compteur = "<<compteur<<endl;
    return mu_f;
}



d_vector energy_pert(int q){
    d_vector e;
    double pi = 3.14159265359;
    for(int i =0; i<int(q/2); i++){
        e.push_back(-4.
                    +(4.*pi/q)*(i+0.5)
                    -(pi/q)*(pi/q)*i*i
                    +(pi/q)*(pi/q)*i
                    -0.5*(pi/q)*(pi/q)
                    +(pi/q)*(pi/q)*(pi/q)*90.*i/180.
                    +(pi/q)*(pi/q)*(pi/q)*15.*i*(i-1)*(i-2)/180.
                    +(pi/q)*(pi/q)*(pi/q)*i*(i-1)*(i-2)*(i-3)/(4.*20736.)
                    -(pi/q)*(pi/q)*(pi/q)*(i+1.)*(i+2.)*(i+3.)*(i+4.)/(4.*20736.)
                    +(pi/q)*(pi/q)*(pi/q)*i*(i-1.)/(18.*20736.)
                    -(pi/q)*(pi/q)*(pi/q)*(i+1.)*(i+2.)/(18.*20736.)
                    -(1260./5040.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*i*(i-1.)
                    -(105./5040.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*i*(i-1.)*(i-2.)*(i-3.)
                    -(70./5040.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    +(3150.*i/1814400.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    +(18900.*i*(i-1.)*(i-2.)/1814400.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    +(945.*i*(i-1.)*(i-2.)*(i-3.)*(i-4.)/1814400.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    -(16632.*i/239500800.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    -(103950.*i*(i-1)/239500800.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    -(277200.*i*(i-1.)*(i-2.)/239500800.)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)*(pi/q)
                    );
    }
   // e.push_back(0);
    return e;
}



cd_vector Matsubara_frequency(double beta, int Nmax, int Nmin){
    dcomp I(0.0, 1.0);
    cd_vector iwn;
    for(int j=Nmin; j <Nmax; j++){
        iwn.push_back(2.0*I*M_PI*(j+0.5)/beta);
    }
    return iwn;
}


