//
//  main.cpp
//  DMFT-IPT
//
//  Created by Shaheen Acheche on 27/09/2016.
//  Copyright © 2016 Shaheen Acheche. All rights reserved.
//

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <complex>
#include <ctime>
#include <vector>
#include "integrator.hpp"
#include "Green.hpp"
#include "Spline.hpp"
#include "config.hpp"


using namespace std;
typedef complex<double> dcomp;
typedef vector<dcomp> cd_vector;
typedef vector<double> d_vector;



int main(int argc, const char * argv[]) {
    //################# Inputs
    parameters param = get_params();
    double U = 1.0;
    d_vector U_tab = linspace(param.U_min,param.U_max,param.nbr_U);
    double mu = U/2.;
    double mu0=0.;
    Spline spli;
    d_vector realasy;
    const int N = 4096;
    int beta=param.beta;
    int compteur = 0;
    int t(0);
    int i;
    double n_target=param.n;

//############################### Variables 
    Green_function G_0(beta, N, -N), G_f(beta,N, -N), G_l(beta, N, -N);
    cd_vector iwn = G_0.get_iwn();
    cd_vector Self;
    d_vector Landau;
    cd_vector Hyb;
    cd_vector I;
    cd_vector denom;
    cd_vector denom1;
    cd_vector  Glat;
    dcomp G1;
//############################# output files
    ofstream delta, docc, spleen, sum, G_final, chemicalpot;
    G_final.precision(20);
    docc.open("Docc.txt");
    chemicalpot.open("chemicalPotsquarebeta_"+to_string(beta)+".dat");
//############################# start

    cout<<"initialization of a guessed Hybridization"<<endl;
    Hyb= Init_hyb(4, iwn);
    cout<<"done!"<<endl;
    mu0 = 0.0;
    while(compteur < U_tab.size()){
        U = U_tab[compteur];
        cout<<"U = "<<U<<endl;
        mu=U/2.;
        G_f.update_mu(mu);
        for(int k=0; k<125; k++){
            cout<<"U = "<<U<<endl;
        G_0.get_new_by_adding(Hyb, mu0);

        spli.define_G_0(G_0, U);
        spli.create_I_tau();
        spli.define();
        I = spli.compute_Iw();
        Self = Compute_self(I, U, 2*n_target, mu, mu0);
        denom.clear();
        denom1.clear();
        for(i = 0; i<G_0.size(); i++){
            denom.push_back(iwn[i]-Hyb[i]);
            denom1.push_back(iwn[i]-Hyb[i]-Self[i]);
        }

        if(abs(density_2ndmethod(mu0, G_0,denom)-n_target)<0.005 && abs(density_2ndmethod(mu, G_f,denom1)-n_target)<0.005){
            t++;
        }else{t=0;}

        if(t ==4){
            cout<<"done!"<<endl;
            t = 0;
            break;
        }
        if(abs(density_2ndmethod(mu0, G_0,denom)-n_target)>0.005){
            mu0=False_pos_get_mu_density(G_0, denom, n_target);
            G_0.update_mu(mu0);
        }

        if(k!=0){
            if(abs(density_2ndmethod(mu, G_f,denom1)-n_target)>0.005){
            mu = False_pos_get_mu_density(G_f, denom1, n_target);
            G_f.update_mu(mu);
                }
            }

        G_f.get_new(denom1);
        cout<<"density of G_f = "<<density(G_f)<<endl;
        cout<<"density of G_0 ="<<density(G_0)<<endl;
        cout<<"mu0 = "<<mu0<<endl;
        cout<<"mu = "<<mu<<endl;
        for(i=0; i<G_0.size(); i++){
            G1 = integrator(*Green_1, -M_PI, M_PI, -M_PI, M_PI, iwn[i]-Self[i]+mu,0.01);
            Hyb[i] = -Self[i] -1.0/G1 +iwn[i]+mu;
        }
    }


        G_final.open("G_lat_p_U"+to_string(U)+"beta_"+to_string(beta)+".txt");
        spleen.open("Selfsquarelatt_U"+to_string(U)+"beta_"+to_string(beta)+".txt");
        //sum.open("Hyb_U"+to_string(U)+"q_"+to_string(q)+"beta_"+to_string(beta)+".txt");
        Glat.clear();
        for(i=0; i<G_0.size(); i++){
                G1 = integrator(*Green_1, -M_PI, M_PI, -M_PI, M_PI, iwn[i]-Self[i]+mu, 0.001);
                Glat.push_back(1./G1);
                if(iwn[i].imag()>0){
                    G_final<<iwn[i].imag()<<"   "<<G1.real()<<" "<<G1.imag()<<endl;
                    spleen<<iwn[i].imag()<<"    "<<Self[i].real()<<"    "<<Self[i].imag()<<endl;
                }

       //     sum<<iwn[i].imag()<<"   "<<Hyb[i].real()<<" "<<Hyb[i].imag()<<endl;
         //   res<<G_0.get_iwn()[i].imag()<<" "<<G_0.iw(i).real()<<" "<<G_0.iw(i).imag()<<endl;
        }
        G_l.get_new(Glat);

        docc<<U<<"  "<<double_occu(Self, G_l, 2*n_target, U)<<endl;
        chemicalpot<<U<<"  "<<mu<<"   "<<mu0<<endl;
        G_final.close();
        sum.close();
        spleen.close();
        compteur ++;
    }

    docc.close();
    chemicalpot.close();
    return 0;

}


